@@ -4,7 +4,7 @@ use crate::PlatformTag;
 use crate::auditwheel::AuditWheelMode;
 use anyhow::{Context, Result};
 use fs_err as fs;
-use pep440_rs::Version;
+use pep440_rs::{Version, VersionSpecifiers};
 use pep508_rs::VersionOrUrl;
 use pyproject_toml::{BuildSystem, Project};
 use serde::{Deserialize, Serialize};
@@ -73,6 +73,37 @@ pub enum GlobPattern {
         /// One or more [Format] values
         format: Formats,
     },
+    /// A glob `path` relative to a crate's OUT_DIR
+    WithOutDir {
+        /// A glob pattern relative to OUT_DIR
+        path: String,
+        /// Source: must be "out-dir"
+        from: IncludeFrom,
+        /// Target path in wheel (e.g. "my_package/")
+        to: String,
+        /// Optional crate name (defaults to the root crate)
+        #[serde(default)]
+        crate_name: Option<String>,
+    },
+}
+
+/// Supported values for the `from` field in include patterns.
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub enum IncludeFrom {
+    /// Include files from the crate's OUT_DIR
+    OutDir,
+}
+
+/// Information about an out-dir include pattern.
+pub struct OutDirInclude<'a> {
+    /// Glob pattern relative to OUT_DIR
+    pub path: &'a str,
+    /// Target path prefix in wheel
+    pub to: &'a str,
+    /// Optional crate name (defaults to root crate)
+    pub crate_name: Option<&'a str>,
 }
 
 impl GlobPattern {
@@ -85,6 +116,25 @@ impl GlobPattern {
                 path,
                 format: formats,
             } if formats.targets(format) => Some(path),
+            // WithOutDir is handled separately, never matched here
+            Self::WithOutDir { .. } => None,
+            _ => None,
+        }
+    }
+
+    /// Returns the out-dir include info if this is a `WithOutDir` pattern.
+    pub fn as_out_dir_include(&self) -> Option<OutDirInclude<'_>> {
+        match self {
+            Self::WithOutDir {
+                path,
+                to,
+                crate_name,
+                ..
+            } => Some(OutDirInclude {
+                path,
+                to,
+                crate_name: crate_name.as_deref(),
+            }),
             _ => None,
         }
     }
@@ -163,6 +213,73 @@ pub enum SdistGenerator {
     Git,
 }
 
+/// SBOM configuration
+#[derive(Serialize, Deserialize, Debug, Clone, Default, PartialEq, Eq)]
+#[serde(rename_all = "kebab-case")]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub struct SbomConfig {
+    /// Generate an SBOM for Rust crates. Defaults to `true`.
+    pub rust: Option<bool>,
+    /// Generate a CycloneDX SBOM for external shared libraries grafted during
+    /// auditwheel repair. Defaults to `true` when repair copies libraries.
+    ///
+    /// The SBOM is written to `<dist-info>/sboms/auditwheel.cdx.json` and
+    /// records which OS packages (deb, rpm, apk) provided the grafted
+    /// libraries, following the same convention as Python's auditwheel.
+    pub auditwheel: Option<bool>,
+    /// Additional SBOM files to include in the `.dist-info/sboms` directory.
+    pub include: Option<Vec<PathBuf>>,
+}
+
+/// A cargo feature specification that can be either a plain feature name
+/// or a conditional feature that is only enabled for certain Python versions.
+///
+/// # Examples
+///
+/// ```toml
+/// [tool.maturin]
+/// features = [
+///   "some-feature",
+///   { feature = "pyo3/abi3-py311", python-version = ">=3.11" },
+///   { feature = "pyo3/abi3-py38", python-version = "<3.11" },
+/// ]
+/// ```
+#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
+#[serde(untagged)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub enum FeatureSpec {
+    /// A plain feature name, always enabled
+    Plain(String),
+    /// A feature enabled only when the target Python version matches
+    Conditional {
+        /// The cargo feature to enable
+        feature: String,
+        /// PEP 440 version specifier for the target Python version, e.g. ">=3.11"
+        #[serde(rename = "python-version")]
+        #[cfg_attr(feature = "schemars", schemars(with = "String"))]
+        python_version: VersionSpecifiers,
+    },
+}
+
+impl FeatureSpec {
+    /// Resolve which conditional features should be enabled for a given Python version.
+    ///
+    /// Returns the feature names whose `python-version` specifier matches the
+    /// given `(major, minor)` version.
+    pub fn resolve_conditional(
+        conditional_features: &[(String, VersionSpecifiers)],
+        major: usize,
+        minor: usize,
+    ) -> Vec<String> {
+        let python_version = Version::new([major as u64, minor as u64]);
+        conditional_features
+            .iter()
+            .filter(|(_, specifiers)| specifiers.contains(&python_version))
+            .map(|(feature, _)| feature.clone())
+            .collect()
+    }
+}
+
 /// The `[tool.maturin]` section of a pyproject.toml
 #[derive(Serialize, Deserialize, Debug, Clone, Default)]
 #[serde(rename_all = "kebab-case")]
@@ -171,9 +288,13 @@ pub struct ToolMaturin {
     // maturin specific options
     /// Module name, accepts setuptools style import name like `foo.bar`
     pub module_name: Option<String>,
-    /// Include files matching the given glob pattern(s)
+    /// Include files matching the given glob pattern(s).
+    /// Patterns are resolved relative to the directory containing `pyproject.toml`.
+    /// When `python-source` is configured, patterns are also tried relative to
+    /// that directory if no matches are found.
     pub include: Option<Vec<GlobPattern>>,
-    /// Exclude files matching the given glob pattern(s)
+    /// Exclude files matching the given glob pattern(s).
+    /// Patterns are resolved relative to the directory containing `pyproject.toml`.
     pub exclude: Option<Vec<GlobPattern>>,
     /// Bindings type
     pub bindings: Option<String>,
@@ -207,8 +328,10 @@ pub struct ToolMaturin {
     pub profile: Option<String>,
     /// Same as `profile` but for "editable" builds
     pub editable_profile: Option<String>,
-    /// Space or comma separated list of features to activate
-    pub features: Option<Vec<String>>,
+    /// List of features to activate.
+    /// Each entry can be a plain feature name string, or a conditional object
+    /// with `feature` and `python-version` keys.
+    pub features: Option<Vec<FeatureSpec>>,
     /// Activate all available features
     pub all_features: Option<bool>,
     /// Do not activate the `default` feature
@@ -232,6 +355,11 @@ pub struct ToolMaturin {
     // in venv.
     #[serde(default)]
     pub use_base_python: bool,
+    /// SBOM configuration
+    pub sbom: Option<SbomConfig>,
+    /// Include the import library (.dll.lib) in the wheel on Windows
+    #[serde(default)]
+    pub include_import_lib: bool,
 }
 
 /// A pyproject.toml as specified in PEP 517
@@ -248,6 +376,8 @@ pub struct PyProjectToml {
     ///
     /// We use it for `[tool.maturin]`
     pub tool: Option<Tool>,
+    /// PEP 735: Dependency groups
+    pub dependency_groups: Option<pyproject_toml::DependencyGroups>,
 }
 
 impl PyProjectToml {
@@ -364,6 +494,13 @@ impl PyProjectToml {
         self.maturin()?.manifest_path.as_deref()
     }
 
+    /// Returns the value of `[tool.maturin.include-import-lib]` in pyproject.toml
+    pub fn include_import_lib(&self) -> bool {
+        self.maturin()
+            .map(|maturin| maturin.include_import_lib)
+            .unwrap_or_default()
+    }
+
     /// Warn about `build-system.requires` mismatching expectations.
     ///
     /// Having a pyproject.toml without a version constraint is a bad idea
@@ -466,7 +603,7 @@ impl PyProjectToml {
 mod tests {
     use crate::{
         PyProjectToml,
-        pyproject_toml::{Format, Formats, GlobPattern, ToolMaturin},
+        pyproject_toml::{FeatureSpec, Format, Formats, GlobPattern, ToolMaturin},
     };
     use expect_test::expect;
     use fs_err as fs;
@@ -513,7 +650,10 @@ mod tests {
         assert_eq!(maturin.profile.as_deref(), Some("dev"));
         assert_eq!(
             maturin.features,
-            Some(vec!["foo".to_string(), "bar".to_string()])
+            Some(vec![
+                FeatureSpec::Plain("foo".to_string()),
+                FeatureSpec::Plain("bar".to_string()),
+            ])
         );
         assert!(maturin.all_features.is_none());
         assert_eq!(maturin.no_default_features, Some(true));
@@ -727,4 +867,80 @@ mod tests {
         "#]];
         expected.assert_eq(&inner_error.to_string());
     }
+
+    #[test]
+    fn test_resolve_conditional_features() {
+        let specs = vec![
+            FeatureSpec::Conditional {
+                feature: "pyo3/abi3-py311".to_string(),
+                python_version: ">=3.11".parse().unwrap(),
+            },
+            FeatureSpec::Conditional {
+                feature: "pyo3/abi3-py38".to_string(),
+                python_version: "<3.11".parse().unwrap(),
+            },
+            FeatureSpec::Conditional {
+                feature: "fast-buffer".to_string(),
+                python_version: ">=3.11".parse().unwrap(),
+            },
+        ];
+        let conditional: Vec<_> = specs
+            .into_iter()
+            .filter_map(|spec| match spec {
+                FeatureSpec::Conditional {
+                    feature,
+                    python_version,
+                } => Some((feature, python_version)),
+                _ => None,
+            })
+            .collect();
+
+        // Python 3.12 should match >=3.11
+        let resolved = FeatureSpec::resolve_conditional(&conditional, 3, 12);
+        assert_eq!(resolved, vec!["pyo3/abi3-py311", "fast-buffer"]);
+
+        // Python 3.11 should match >=3.11
+        let resolved = FeatureSpec::resolve_conditional(&conditional, 3, 11);
+        assert_eq!(resolved, vec!["pyo3/abi3-py311", "fast-buffer"]);
+
+        // Python 3.9 should match <3.11
+        let resolved = FeatureSpec::resolve_conditional(&conditional, 3, 9);
+        assert_eq!(resolved, vec!["pyo3/abi3-py38"]);
+
+        // Python 3.8 should match <3.11
+        let resolved = FeatureSpec::resolve_conditional(&conditional, 3, 8);
+        assert_eq!(resolved, vec!["pyo3/abi3-py38"]);
+    }
+
+    #[test]
+    fn test_feature_spec_deserialize_mixed() {
+        let toml_str = r#"
+            features = [
+                "plain-feature",
+                { feature = "pyo3/abi3-py311", python-version = ">=3.11" },
+            ]
+        "#;
+        let maturin: ToolMaturin = toml::from_str(toml_str).unwrap();
+        assert_eq!(
+            maturin.features,
+            Some(vec![
+                FeatureSpec::Plain("plain-feature".to_string()),
+                FeatureSpec::Conditional {
+                    feature: "pyo3/abi3-py311".to_string(),
+                    python_version: ">=3.11".parse().unwrap(),
+                },
+            ])
+        );
+    }
+
+    #[test]
+    fn test_feature_spec_deserialize_invalid_specifier() {
+        let toml_str = r#"
+            features = [
+                { feature = "foo", python-version = "not-a-version" },
+            ]
+        "#;
+        let result: Result<ToolMaturin, _> = toml::from_str(toml_str);
+        assert!(result.is_err());
+    }
 }
